@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -26,6 +26,8 @@
 # When updating this file, consider if lint.py or warp.py also need
 # updating.
 
+from __future__ import print_function
+
 import renpy.display
 import renpy.test
 
@@ -191,14 +193,37 @@ class ArgumentInfo(object):
 
         return tuple(args), kwargs
 
+    def get_code(self):
+
+        l = [ ]
+
+        for keyword, expression in self.arguments:
+            if keyword is not None:
+                l.append("{}={}".format(keyword, expression))
+            else:
+                l.append(expression)
+
+        if self.extrapos is not None:
+            l.append("*" + self.extrapos)
+
+        if self.extrakw is not None:
+            l.append("**" + self.extrakw)
+
+        return "(" + ", ".join(l) + ")"
+
 
 def __newobj__(cls, *args):
     return cls.__new__(cls, *args)
 
-# This represents a string containing python code.
+
+# A list of pyexprs that need to be precompiled.
+pyexpr_list = [ ]
 
 
 class PyExpr(unicode):
+    """
+    Represents a string containing python code.
+    """
 
     __slots__ = [
         'filename',
@@ -210,6 +235,10 @@ class PyExpr(unicode):
         self.filename = filename
         self.linenumber = linenumber
 
+        # Queue the string for precompilation.
+        if self and (renpy.game.script.all_pyexpr is not None):
+            renpy.game.script.all_pyexpr.append(self)
+
         return self
 
     def __getnewargs__(self):
@@ -302,7 +331,10 @@ class Scry(object):
         if self._next is None:
             return None
         else:
-            return self._next.scry()
+            try:
+                return self._next.scry()
+            except:
+                return None
 
 
 class Node(object):
@@ -330,6 +362,13 @@ class Node(object):
     # take_translations.
     translation_relevant = False
 
+    # How does the node participate in rollback?
+    #
+    # * "normal" in normal mode.
+    # * "never" generally never.
+    # * "force" force it to start.
+    rollback = "normal"
+
     def __init__(self, loc):
         """
         Initializes this Node object.
@@ -463,6 +502,15 @@ class Node(object):
         # Does nothing by default.
         return
 
+    warp = False
+
+    def can_warp(self):
+        """
+        Returns true if this should be run while warping, False otherwise.
+        """
+
+        return self.warp
+
 
 def say_menu_with(expression, callback):
     """
@@ -484,6 +532,9 @@ def say_menu_with(expression, callback):
         callback(what)
 
 
+fast_who_pattern = re.compile(r'[a-zA-Z_][a-zA-Z_0-9]*$')
+
+
 def eval_who(who, fast=None):
     """
     Evaluates the `who` parameter to a say statement.
@@ -493,7 +544,7 @@ def eval_who(who, fast=None):
         return None
 
     if fast is None:
-        fast = bool(re.match(renpy.parser.word_regexp + "$", who))
+        fast = bool(fast_who_pattern.match(who))
 
     if fast:
 
@@ -582,6 +633,9 @@ class Say(Node):
             rv.append("with")
             rv.append(self.with_)
 
+        if self.arguments:
+            rv.append(self.arguments.get_code())
+
         return " ".join(rv)
 
     def execute(self):
@@ -655,13 +709,7 @@ class Say(Node):
     def scry(self):
         rv = Node.scry(self)
 
-        if self.who is not None:
-            if self.who_fast:
-                who = getattr(renpy.store, self.who)
-            else:
-                who = renpy.python.py_eval(self.who)
-        else:
-            who = None
+        who = eval_who(self.who, self.who_fast)
 
         if self.interact:
             renpy.exports.scry_say(who, rv)
@@ -716,6 +764,8 @@ class Init(Node):
 
 class Label(Node):
 
+    rollback = "force"
+
     translation_relevant = True
 
     __slots__ = [
@@ -810,7 +860,12 @@ class Python(Node):
         super(Python, self).__init__(loc)
 
         self.hide = hide
-        self.code = PyCode(python_code, loc=loc, mode='exec')
+
+        if hide:
+            self.code = PyCode(python_code, loc=loc, mode='hide')
+        else:
+            self.code = PyCode(python_code, loc=loc, mode='exec')
+
         self.store = store
 
     def diff_info(self):
@@ -861,7 +916,12 @@ class EarlyPython(Node):
         super(EarlyPython, self).__init__(loc)
 
         self.hide = hide
-        self.code = PyCode(python_code, loc=loc, mode='exec')
+
+        if hide:
+            self.code = PyCode(python_code, loc=loc, mode='hide')
+        else:
+            self.code = PyCode(python_code, loc=loc, mode='exec')
+
         self.store = store
 
     def diff_info(self):
@@ -1068,6 +1128,8 @@ class Show(Node):
         'atl',
         ]
 
+    warp = True
+
     def __init__(self, loc, imspec, atl=None):
         """
         @param imspec: A triple consisting of an image name (itself a
@@ -1099,6 +1161,8 @@ class Show(Node):
 
 class ShowLayer(Node):
 
+    warp = True
+
     __slots__ = [
         'layer',
         'at_list',
@@ -1143,6 +1207,8 @@ class Scene(Node):
         'atl',
         ]
 
+    warp = True
+
     def __init__(self, loc, imgspec, layer, atl=None):
         """
         @param imspec: A triple consisting of an image name (itself a
@@ -1193,6 +1259,8 @@ class Hide(Node):
         'imspec',
         ]
 
+    warp = True
+
     def __init__(self, loc, imgspec):
         """
         @param imspec: A triple consisting of an image name (itself a
@@ -1722,8 +1790,13 @@ class UserStatement(Node):
         self.code_block = None
         self.parsed = None
 
-        # Do not store the parse quite yet.
-        _parse_info = renpy.statements.parse(self, self.line, self.block)
+        self.name = self.call("label")
+
+        # Do not store the parse.
+        self.parsed = None
+
+    def __repr__(self):
+        return "<UserStatement {!r}>".format(self.line)
 
     def get_children(self, f):
         f(self)
@@ -1812,6 +1885,13 @@ class UserStatement(Node):
     def get_code(self, dialogue_filter=None):
         return self.line
 
+    def can_warp(self):
+
+        if self.call("warp"):
+            return True
+
+        return False
+
 
 def create_store(name):
     if name not in renpy.config.special_namespaces:
@@ -1843,6 +1923,8 @@ def get_namespace(store):
 # and then again at init time.
 EARLY_CONFIG = { "save_directory" }
 
+define_statements = [ ]
+
 
 class Define(Node):
 
@@ -1879,6 +1961,8 @@ class Define(Node):
         next_node(self.next)
         statement_name("define")
 
+        define_statements.append(self)
+
         value = renpy.python.py_eval_bytecode(self.code.bytecode)
 
         if self.store == 'store':
@@ -1890,6 +1974,25 @@ class Define(Node):
         ns, _special = get_namespace(self.store)
         ns.set(self.varname, value)
 
+    def redefine(self, stores):
+
+        if self.store not in stores:
+            return
+
+        value = renpy.python.py_eval_bytecode(self.code.bytecode)
+        ns, _special = get_namespace(self.store)
+        ns.set(self.varname, value)
+
+
+def redefine(stores):
+    """
+    Re-runs the given define statements.
+    """
+
+    for i in define_statements:
+        i.redefine(stores)
+
+
 
 # All the default statements, in the order they were registered.
 default_statements = [ ]
@@ -1952,7 +2055,10 @@ class Default(Node):
             d.ever_been_changed.add("_defaults_set")
 
         if self.varname not in defaults_set:
-            d[self.varname] = renpy.python.py_eval_bytecode(self.code.bytecode)
+
+            if start or (self.varname not in d.ever_been_changed):
+                d[self.varname] = renpy.python.py_eval_bytecode(self.code.bytecode)
+
             d.ever_been_changed.add(self.varname)
 
             defaults_set.add(self.varname)
@@ -2013,19 +2119,23 @@ class Translate(Node):
     goes to the end of the translate statement in the None language.
     """
 
+    rollback = "never"
+
     translation_relevant = True
 
     __slots__ = [
         "identifier",
+        "alternate",
         "language",
         "block",
         "after",
         ]
 
-    def __init__(self, loc, identifier, language, block):
+    def __init__(self, loc, identifier, language, block, alternate=None):
         super(Translate, self).__init__(loc)
 
         self.identifier = identifier
+        self.alternate = alternate
         self.language = language
         self.block = block
 
@@ -2050,6 +2160,9 @@ class Translate(Node):
         if self.after is old:
             self.after = new
 
+    def lookup(self):
+        return renpy.game.script.translator.lookup_translate(self.identifier, getattr(self, "alternate", None))
+
     def execute(self):
 
         statement_name("translate")
@@ -2063,18 +2176,18 @@ class Translate(Node):
             renpy.game.seen_translates_count += 1
             renpy.game.new_translates_count += 1
 
-        next_node(renpy.game.script.translator.lookup_translate(self.identifier))
+        next_node(self.lookup())
 
         renpy.game.context().translate_identifier = self.identifier
-        renpy.game.context().translate_block_language = self.language
+        renpy.game.context().alternate_translate_identifier = getattr(self, "alternate", None)
 
     def predict(self):
-        node = renpy.game.script.translator.lookup_translate(self.identifier)
+        node = self.lookup()
         return [ node ]
 
     def scry(self):
         rv = Scry()
-        rv._next = renpy.game.script.translator.lookup_translate(self.identifier)
+        rv._next = self.lookup()
         return rv
 
     def get_children(self, f):
@@ -2093,6 +2206,8 @@ class EndTranslate(Node):
     resetting the translation identifier.
     """
 
+    rollback = "never"
+
     def __init__(self, loc):
         super(EndTranslate, self).__init__(loc)
 
@@ -2104,7 +2219,7 @@ class EndTranslate(Node):
         statement_name("end translate")
 
         renpy.game.context().translate_identifier = None
-        renpy.game.context().translate_block_language = None
+        renpy.game.context().alternate_translate_identifier = None
 
 
 class TranslateString(Node):
