@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -23,8 +23,7 @@
 # This includes both simple settings (like the screen dimensions) and
 # methods that perform standard tasks, like the say and menu methods.
 
-# This will be deleted by the end of this file.
-import renpy.display
+from __future__ import print_function
 
 import collections
 import os
@@ -93,7 +92,10 @@ savedir = None
 
 # The number of screens worth of images that are allowed to
 # live in the image cache at once.
-image_cache_size = 16
+image_cache_size = None
+
+# The size of the image cache, in megabytes.
+image_cache_size_mb = 300
 
 # The number of statements we will analyze when doing predictive
 # loading. Please note that this is a total number of statements in a
@@ -246,9 +248,12 @@ editor_file_separator = None  # os.environ.get('RENPY_EDITOR_FILE_SEPARATOR', '"
 # Enable developer mode?
 developer = False  # Changed to True or False in the init code.
 
-# The original value of config.developer.
+# The value of developer requested by the creator (True, False, or "auto")
 original_developer = False
 
+# The default value of developer that's used when it's set to auto.
+default_developer = False
+
 # A logfile that logging messages are sent to.
 log = None
 
@@ -302,13 +307,13 @@ autosave_slots = 10
 autosave_frequency = int(os.environ.get("RENPY_AUTOSAVE_FREQUENCY", "200"))
 
 # The callback that is used by the scene statement.
-scene = renpy.exports.scene
+scene = None
 
 # The callback that is used by the show statement.
-show = renpy.exports.show
+show = None
 
 # The callback that is used by the hide statement.
-hide = renpy.exports.hide
+hide = None
 
 # Should we use cPickle or pickle for load/save?
 use_cpickle = True
@@ -435,6 +440,9 @@ new_character_image_argument = True
 # corresponding to a shown image.
 say_attribute_transition = None
 
+# The layer the say_attribute_transition runs on.
+say_attribute_transition_layer = None
+
 # What is the name and version of this game?
 name = ""
 version = ""
@@ -516,7 +524,7 @@ choice_layer = "screens"
 raw_tracebacks = ("RENPY_RAW_TRACEBACKS" in os.environ)
 
 # A function to process texts which should be spoken
-tts_function = renpy.display.tts.default_tts_function
+tts_function = None
 
 # Channels that stop voice playback.
 tts_voice_channels = [ "voice" ]
@@ -751,9 +759,167 @@ profile_screens = [ ]
 # Should Ren'Py search for system fonts.
 allow_sysfonts = False
 
-del renpy
+# Should Ren'Py tightly loop during fadeouts? (That is, not stop the fadeout
+# if it reaches the end of a trac.)
+tight_loop_default = True
+
+# Should Ren'Py apply style_prefix to viewport scrollbar styles?
+prefix_viewport_scrollbar_styles = True
+
+# These functions are called to determine if Ren'Py needs to redraw the screen.
+needs_redraw_callbacks = [ ]
+
+# Should a hyperlink inherit the size of the text its in?
+hyperlink_inherit_size = True
+
+# A list of callbacks that are called when a line is printed to stdout or
+# stderr.
+stdout_callbacks = [ ]
+stderr_callbacks = [ ]
+
+# Should ATL automatically cause polar motion when angle changes.
+automatic_polar_motion = True
+
+# Functions that are called to generate lint stats.
+lint_stats_callbacks = [ ]
+
+# Should we apply position properties to the side of a viewport?
+position_viewport_side = True
+
+# Things that be given properties via Character.
+character_id_prefixes = [ ]
+
+# Should {nw} wait for voice.
+nw_voice = True
+
+# If not None, a function that's used to process say arguments.
+say_arguments_callback = None
+
+# Should we show an atl interpolation for one frame?
+atl_one_frame = True
+
+# Should we keep the show layer state?
+keep_show_layer_state = True
+
+# A list of callbacks that are called when fast skipping happens.
+fast_skipping_callbacks = [ ]
+
+# Should the audio periodic callback run in its own thread.
+audio_periodic_thread = True
+
+# A list of fonts to preload on Ren'Py startup.
+preload_fonts = [ ]
+
+# Should Ren'Py process multiple ATL events in a single update?
+atl_multiple_events = True
+
+# A callback that's called when checking to see if a file is loadable.
+loadable_callback = None
+
+# How many frames should be drawn fast each time the screen needs to be
+# updated?
+fast_redraw_frames = 4
+
+# The color passed to glClearColor when clearing the screen.
+gl_clear_color = "#000"
+
+# Screens that are updated once per frame rather than once per interaction.
+per_frame_screens = [ ]
+
+# How long we store performance data for.
+performance_window = 5.0
+
+# How long does a frame have to take (to the event) to trigger profiling.
+profile_time = 1.0 / 50.0
+
+# What event do we check to see if the profile needs to be printed?
+profile_to_event = "flip"
+
+# Should we instantly zap transient displayables, or properly hide them?
+fast_unhandled_event = True
+
+# Should a fast path be used when displaying empty windows.
+fast_empty_window = True
+
+# Should all nodes participate in rollback?
+all_nodes_rollback = False
+
+# Should Ren'Py manage GC itself?
+manage_gc = True
+
+# Default thresholds that apply to garbage collection.
+gc_thresholds = (25000, 10, 10)
+
+# The threshold for a level 0 gc when we have the time.
+idle_gc_count = 2500
+
+# Should we print unreachable.
+gc_print_unreachable = "RENPY_GC_PRINT_UNREACHABLE" in os.environ
+
+# The first frame that we consider to be "idle", so we can do gc and
+# prediction.
+idle_frame = 4
+
+# Does taking the transform state go through image reference targets?
+take_state_from_target = False
+
+# Does ui.viewport set the child_size if not set?
+scrollbar_child_size = True
+
+# Should surfaces be cached?
+cache_surfaces = False
+
+# Should we optimize textures by taking the bounding rect?
+optimize_texture_bounds = True
+
+# Should we predict everything in a ConditionSwitch?
+conditionswitch_predict_all = False
+
+# Transform events to deliver each time one happens.
+repeat_transform_events = [ "show", "replace", "update" ]
+
+# How many statements should we warp through?
+warp_limit = 1000
+
+# Should dissolve statments force the use of alpha.
+dissolve_force_alpha = True
+
+# A map from a displayable prefix to a function that returns a displayable
+# corresponding to the argument.
+displayable_prefix = { }
+
+# Should we re-play a movie when it's shown again.
+replay_movie_sprites = True
+
+# A callback that is called when entering a new context.
+context_callback = None
+
+# Should we reject . and .. in filenames?
+reject_relative = True
+
+# The prefix to use on the side image.
+side_image_prefix_tag = 'side'
+
+# Do the say attributes of a hidden side image use the side image tag?
+say_attributes_use_side_image = True
+
 del os
+del collections
 
 
 def init():
-    pass
+    import renpy.display
+
+    global scene
+    scene = renpy.exports.scene
+
+    global show
+    show = renpy.exports.show
+
+    global hide
+    hide = renpy.exports.hide
+
+    global tts_function
+    tts_function = renpy.display.tts.default_tts_function
+
+    import os
