@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -23,6 +23,8 @@
 # Most of the guts of this file have been moved into im.py, with only some
 # of the stuff thar uses images remaining.
 
+from __future__ import print_function
+
 import renpy.display
 import renpy.text
 from renpy.display.render import render, Render
@@ -35,7 +37,21 @@ images = { }
 
 # A map from image tag to lists of possible attributes for images with that
 # tag.
-image_attributes = collections.defaultdict(list)
+image_attributes = collections.defaultdict(dict)
+
+# The set of image names Ren'Py knows about, as strings with spaces.
+image_names = [ ]
+
+
+def list_images():
+    """
+    :doc: image_func
+
+    Returns a list of images that have been added to Ren'Py, as a list of
+    strings with spaces between the name components.
+    """
+
+    return image_names
 
 
 def get_available_image_tags():
@@ -97,12 +113,24 @@ def check_image_attributes(tag, attributes):
     Otherwise, returns None.
     """
 
-    ca = get_tag_method(tag, "_choose_attributes")
+    l = [ ]
 
-    if ca is not None:
-        return ca(tag, attributes, None)
+    for attrs, d in image_attributes[tag].items():
 
-    l = get_available_image_attributes(tag, attributes)
+        remainder = [ i for i in attributes if i not in attrs ]
+
+        ca = getattr(d, "_choose_attributes", None)
+
+        if ca is not None:
+
+            chosen = ca(tag, remainder, None)
+            if chosen is not None:
+                l.append(list(attrs) + list(chosen))
+
+        else:
+
+            if not remainder:
+                l.append(attrs)
 
     # Check to see if there's an image that is exactly the one we want.
     for i in l:
@@ -133,32 +161,81 @@ def get_ordered_image_attributes(tag, attributes=(), sort=None):
 
     """
 
-    la = get_tag_method(tag, "_list_attributes")
-
-    if la is not None:
-        return la(tag, attributes)
+    sequences = [ ]
 
     attrcount = collections.defaultdict(int)
     attrtotalpos = collections.defaultdict(float)
 
-    for attrlist in get_available_image_attributes(tag, attributes):
-        for i, attr in enumerate(attrlist):
-            attrcount[attr] += 1
-            attrtotalpos[attr] += i
+    for attrs, d in sorted(image_attributes[tag].items()):
+
+        la = getattr(d, "_list_attributes", None)
+        if la is not None:
+
+            sequence = list(attrs) + la(tag, [ i for i in attributes if i not in attrs ])
+
+            if not all(i in sequence for i in attributes):
+                continue
+
+            sequences.append(sequence)
+
+        else:
+
+            if not all(i in attrs for i in attributes):
+                continue
+
+            for i, attr in enumerate(attrs):
+                attrcount[attr] += 1
+                attrtotalpos[attr] += i
 
     if sort is None:
+        return list(set(attrcount.keys()) | set(j for i in sequences for j in i))
 
-        return list(attrcount.keys())
+    # If we have a sequence, do a topological sort on the before-after relation -
+    # with an ajustment to make sure it will complete even if it loops.
 
-    else:
+    rv = [ ]
 
-        l = [ ]
+    # A map from an attribute to all the attributes it is after.
+    after = collections.defaultdict(set)
 
-        for attr in attrcount:
+    for i in sequences:
+        while i:
+
+            j = i.pop(0)
+
+            # Ensure it exists.
+            after[j]
+
+            for k in i:
+                after[k].add(j)
+
+    while after:
+
+        mincount = min(len(i) for i in after.values())
+        ready = set(k for k, v in after.items() if len(v) == mincount)
+
+        for i in ready:
+            del after[i]
+
+        for k in after:
+            after[k] = after[k] - ready
+
+        ready = list(ready)
+        ready.sort(key=lambda a : (sort(a), a))
+
+        rv.extend(ready)
+
+    l = [ ]
+
+    for attr in attrcount:
+        if attr not in rv:
             l.append((attrtotalpos[attr] / attrcount[attr], sort(attr), attr))
 
-        l.sort()
-        return [ i[2] for i in l ]
+    l.sort()
+    for i in l:
+        rv.append(i[2])
+
+    return rv
 
 
 def register_image(name, d):
@@ -174,7 +251,9 @@ def register_image(name, d):
     rest = name[1:]
 
     images[name] = d
-    image_attributes[tag].append(rest)
+    image_attributes[tag][rest] = d
+
+    image_names.append(" ".join(name))
 
 
 def image_exists(name, exact=False):
@@ -209,6 +288,20 @@ def image_exists(name, exact=False):
     return False
 
 
+def get_registered_image(name):
+    """
+    :doc: image_func
+
+    If an image with the same name has been registered with renpy.register_image,
+    returns it. Otherwise, returns None.
+    """
+
+    if not isinstance(name, tuple):
+        name = tuple(name.split())
+
+    return images.get(name)
+
+
 def wrap_render(child, w, h, st, at):
     rend = render(child, w, h, st, at)
     rv = Render(rend.width, rend.height)
@@ -303,6 +396,12 @@ class ImageReference(renpy.display.core.Displayable):
             error("Image '%s' not found." % ' '.join(self.name))
             return False
 
+        if name and (self._args.name == name):
+            error("Image '{}' refers to itself.".format(' '.join(name)))
+            return False
+
+        args += self._args.args
+
         try:
 
             a = self._args.copy(name=name, args=args)
@@ -367,6 +466,12 @@ class ImageReference(renpy.display.core.Displayable):
         rv.target = target
         return rv
 
+    def _handles_event(self, event):
+        if self.target is None:
+            return False
+
+        return self.target._handles_event(event)
+
     def _hide(self, st, at, kind):
         if self.target is None:
             self.find_target()
@@ -442,6 +547,9 @@ class DynamicImage(renpy.display.core.Displayable):
     # Have we been locked, so we never change?
     locked = False
 
+    # The name used for hashing.
+    hash_name = None
+
     def __init__(self, name, scope=None, **properties):
         super(DynamicImage, self).__init__(**properties)
 
@@ -468,7 +576,14 @@ class DynamicImage(renpy.display.core.Displayable):
         return u"DynamicImage {!r}".format(self.name)
 
     def __hash__(self):
-        return hash(self.name)
+
+        if self.hash_name is None:
+            if isinstance(self.name, list):
+                self.hash_name = tuple(self.name)
+            else:
+                self.hash_name = self.name
+
+        return hash(self.hash_name)
 
     def __eq__(self, o):
         if self is o:
@@ -563,6 +678,12 @@ class DynamicImage(renpy.display.core.Displayable):
         rv.locked = True
         return rv
 
+    def _handles_event(self, event):
+        if self.target is None:
+            return False
+
+        return self.target._handles_event(event)
+
     def _hide(self, st, at, kind):
         if self.target is None:
             self.find_target()
@@ -693,6 +814,14 @@ class ShownImageInfo(renpy.object.Object):
 
         return { t for l, t in self.shown if l == layer }
 
+    def get_hidden_tags(self, layer):
+        """
+        Returns the set of tags on layer that have attributes,
+        but aren't being shown.
+        """
+
+        return { t for l, t in self.attributes if l == layer if (l, t) not in self.shown }
+
     def predict_scene(self, layer):
         """
         Predicts the scene statement being called on layer.
@@ -742,7 +871,10 @@ class ShownImageInfo(renpy.object.Object):
 
         # If the name matches one that exactly exists, return it.
         if (name in images) and not (wanted or remove):
-            return name
+            ca = getattr(images[name], "_choose_attributes", None)
+
+            if ca is None:
+                return name
 
         nametag = name[0]
 
@@ -765,20 +897,25 @@ class ShownImageInfo(renpy.object.Object):
 
     def choose_image(self, tag, required, optional, exception_name):
 
-        ca = get_tag_method(tag, "_choose_attributes")
-        if ca is not None:
-            attrs = ca(tag, required, optional)
-
-            if attrs is not None:
-                return (tag,) + attrs
-
         # The longest length of an image that matches.
         max_len = -1
 
         # The list of matching images.
         matches = None
 
-        for attrs in image_attributes[tag]:
+        for attrs, d in image_attributes[tag].items():
+
+            ca = getattr(d, "_choose_attributes", None)
+
+            if ca:
+                ca_required = { i for i in required if i not in attrs }
+                ca_optional = { i for i in optional if i not in attrs }
+                newattrs = ca(tag, ca_required, ca_optional)
+
+                if newattrs is None:
+                    continue
+
+                attrs = attrs + newattrs
 
             num_required = 0
 
