@@ -87,7 +87,10 @@ from renpy.python import rng as random
 from renpy.atl import atl_warper
 from renpy.easy import predict, displayable, split_properties
 from renpy.parser import unelide_filename, get_parse_errors
+
 from renpy.translation import change_language, known_languages
+from renpy.translation.generation import generic_filter as transform_text
+
 
 from renpy.persistent import register_persistent
 
@@ -158,6 +161,7 @@ def public_api():
     displayable, split_properties
     unelide_filename, get_parse_errors
     change_language, known_languages
+    transform_text
     language_tailor
     register_persistent
     register_statement
@@ -337,7 +341,7 @@ def image(name, d):
     """
     :doc: se_images
 
-    Defines an image. This function is the python equivalent of the
+    Defines an image. This function is the Python equivalent of the
     image statement.
 
     `name`
@@ -462,7 +466,7 @@ def showing(name, layer='master'):
     :doc: image_func
 
     Returns true if an image with the same tag as `name` is showing on
-    `layer`
+    `layer`.
 
     `image`
         May be a string giving the image name or a tuple giving each
@@ -709,7 +713,7 @@ def hide(name, layer=None):
     """
     :doc: se_images
 
-    Hides an image from a layer. The python equivalent of the hide statement.
+    Hides an image from a layer. The Python equivalent of the hide statement.
 
     `name`
          The name of the image to hide. Only the image tag is used, and
@@ -1404,7 +1408,7 @@ def with_statement(trans, always=False, paired=None, clear=True):
     :name: renpy.with_statement
     :args: (trans, always=False)
 
-    Causes a transition to occur. This is the python equivalent of the
+    Causes a transition to occur. This is the Python equivalent of the
     with statement.
 
     `trans`
@@ -1448,8 +1452,7 @@ def with_statement(trans, always=False, paired=None, clear=True):
 
         trans = trans[None]
 
-    else:
-        return renpy.game.interface.do_with(trans, paired, clear=clear)
+    return renpy.game.interface.do_with(trans, paired, clear=clear)
 
 
 globals()["with"] = with_statement
@@ -1627,7 +1630,7 @@ def reload_script():
         utter_restart()
 
 
-def quit(relaunch=False, status=0):  # @ReservedAssignment
+def quit(relaunch=False, status=0, save=False):  # @ReservedAssignment
     """
     :doc: other
 
@@ -1639,8 +1642,18 @@ def quit(relaunch=False, status=0):  # @ReservedAssignment
     `status`
         The status code Ren'Py will return to the operating system.
         Generally, 0 is success, and positive integers are failure.
+
+    `save`
+        If true, the game is saved in :var:`_quit_slot` before Ren'Py
+        terminates.
     """
 
+    if save and (renpy.store._quit_slot is not None):
+        renpy.loadsave.save(renpy.store._quit_slot, getattr(renpy.store, "save_name", ""))
+
+    if has_label("quit"):
+        call_in_new_context("quit")
+
     raise renpy.game.QuitException(relaunch=relaunch, status=status)
 
 
@@ -1761,11 +1774,11 @@ def transition(trans, layer=None, always=False, force=False):
     """
 
     if isinstance(trans, dict):
-        for k, v in trans.items():
-            trans(k, v, always=always, force=force)
+        for layer, t in trans.items():
+            transition(t, layer=layer, always=always, force=force)
         return
 
-    if not always and not renpy.game.preferences.transitions:
+    if (not always) and not renpy.game.preferences.transitions:
         trans = None
 
     renpy.game.interface.set_transition(trans, layer, force=force)
@@ -2084,7 +2097,7 @@ def file(fn):  # @ReservedAssignment
     accessed using Ren'Py's standard search method, and may reside in an RPA archive.
     or as an Android asset.
 
-    The object supports a wide subset of the fields and methods found on python's
+    The object supports a wide subset of the fields and methods found on Python's
     standard file object, opened in binary mode. (Basically, all of the methods that
     are sensible for a read-only file.)
     """
@@ -2149,7 +2162,7 @@ def show_layer_at(at_list, layer='master', reset=True):
     :doc: se_images
     :name: renpy.show_layer_at
 
-    The python equivalent of the ``show layer`` `layer` ``at`` `at_list`
+    The Python equivalent of the ``show layer`` `layer` ``at`` `at_list`
     statement.
 
     `reset`
@@ -2436,7 +2449,7 @@ def game_menu(screen=None):
     if screen is None:
         call_in_new_context("_game_menu")
     else:
-        call_in_new_context("_game_menu", screen)
+        call_in_new_context("_game_menu", _game_menu_screen=screen)
 
 
 def shown_window():
@@ -2858,7 +2871,7 @@ def variant(name):
 
     Returns true if a `name` is a screen variant that can be chosen
     by Ren'Py. See :ref:`screen-variants` for more details. This function
-    can be used as the condition in a python if statement to set up the
+    can be used as the condition in a Python if statement to set up the
     appropriate styles for the selected screen variant.
 
     `name` can also be a list of variants, in which case this function
@@ -2903,7 +2916,7 @@ def get_say_attributes():
     return renpy.game.context().say_attributes
 
 
-def get_side_image(prefix_tag, image_tag=None, not_showing=True, layer='master'):
+def get_side_image(prefix_tag, image_tag=None, not_showing=True, layer=None):
     """
     :doc: other
 
