@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,6 +19,8 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import print_function
+
 import pygame_sdl2 as pygame
 
 try:
@@ -64,7 +66,7 @@ class ImageFont(object):
             g = textsupport.Glyph()  # @UndefinedVariable
 
             g.character = ord(c)
-            g.ascent = self.height
+            g.ascent = self.baseline
             g.line_spacing = self.height
 
             width = self.width.get(c, None)
@@ -117,7 +119,8 @@ class SFont(ImageFont):
                  spacewidth,
                  default_kern,
                  kerns,
-                 charset):
+                 charset,
+                 baseline=None):
 
         super(SFont, self).__init__()
 
@@ -126,6 +129,7 @@ class SFont(ImageFont):
         self.default_kern = default_kern
         self.kerns = kerns
         self.charset = charset
+        self.baseline = baseline
 
     def load(self):
 
@@ -140,7 +144,11 @@ class SFont(ImageFont):
         sw, sh = surf.get_size()
         height = sh
         self.height = height  # W0201
-        self.baseline = height  # W0201
+        if self.baseline is None:
+            self.baseline = height  # W0201
+        elif self.baseline < 0:
+            # Negative value is the distance from the bottom (vs top)
+            self.baseline = height + self.baseline  # W0201
 
         # Create space characters.
         self.chars[u' '] = renpy.display.pgrender.surface((self.spacewidth, height), True)
@@ -391,7 +399,7 @@ class ScaledImageFont(ImageFont):
 
 
 def register_sfont(name=None, size=None, bold=False, italics=False, underline=False,
-                   filename=None, spacewidth=10, default_kern=0, kerns={},
+                   filename=None, spacewidth=10, baseline=None, default_kern=0, kerns={},
                    charset=u"!\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"):
     """
     :doc: image_fonts
@@ -423,6 +431,13 @@ def register_sfont(name=None, size=None, bold=False, italics=False, underline=Fa
     `spacewidth`
         The width of a space character, an integer in pixels.
 
+    `baseline`
+        The distance from the top of the font to the baseline (the invisible
+        line letters sit on), an integer in pixels.  If this font is mixed with
+        other fonts, their baselines will be aligned.  Negative values indicate
+        distance from the bottom of the font instead, and ``None`` means the
+        baseline equals the height (i.e., is at the very bottom of the font).
+
     `default_kern`
         The default kern spacing between characters, in pixels.
 
@@ -430,7 +445,8 @@ def register_sfont(name=None, size=None, bold=False, italics=False, underline=Fa
         A map from two-character strings to the kern that should be used between
         those characters.
 
-    `charset` - The character set of the font. A string containing characters in
+    `charset`
+        The character set of the font. A string containing characters in
         the order in which they are found in the image. The default character
         set for a SFont is::
 
@@ -442,7 +458,7 @@ def register_sfont(name=None, size=None, bold=False, italics=False, underline=Fa
     if name is None or size is None or filename is None:
         raise Exception("When registering an SFont, the font name, font size, and filename are required.")
 
-    sf = SFont(filename, spacewidth, default_kern, kerns, charset)
+    sf = SFont(filename, spacewidth, default_kern, kerns, charset, baseline)
     image_fonts[(name, size, bold, italics)] = sf
 
 
