@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,6 +19,8 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import print_function
+
 import math
 import renpy.display
 
@@ -183,6 +185,7 @@ class TextSegment(object):
             self.ruby_top = source.ruby_top
             self.ruby_bottom = source.ruby_bottom
             self.hinting = source.hinting
+            self.outline_color = source.outline_color
 
         else:
             self.hyperlink = 0
@@ -220,6 +223,7 @@ class TextSegment(object):
         self.black_color = style.black_color
         self.hyperlink = None
         self.kerning = layout.scale(style.kerning)
+        self.outline_color = None
 
         if style.slow_cps is True:
             self.cps = renpy.game.preferences.text_cps
@@ -246,6 +250,8 @@ class TextSegment(object):
 
         if self.ruby_bottom:
             textsupport.mark_ruby_bottom(rv)
+        elif self.ruby_top == "alt":
+            textsupport.mark_altruby_top(rv)
         elif self.ruby_top:
             textsupport.mark_ruby_top(rv)
 
@@ -257,7 +263,7 @@ class TextSegment(object):
         """
 
         if di.override_color:
-            color = di.override_color
+            color = self.outline_color or di.override_color
             black_color = None
         else:
             color = self.color
@@ -466,7 +472,10 @@ class Layout(object):
             Layout (which must be another Layout of the same text).
         """
 
-        if drawable_res and renpy.config.drawable_resolution_text:
+        width = min(32767, width)
+        height = min(32767, height)
+
+        if drawable_res and (not size_only) and renpy.config.drawable_resolution_text:
             # How much do we want to oversample the text by, compared to the
             # virtual resolution.
             self.oversample = renpy.display.draw.draw_per_virt
@@ -539,19 +548,30 @@ class Layout(object):
         # The time at which the next glyph will be displayed.
         gt = 0.0
 
-        # True if we've encountered the end segment while assigning times.
-        ended = False
-
         # 2. Breaks the text into a list of paragraphs, where each paragraph is
         # represented as a list of (Segment, text string) tuples.
         #
         # This takes information from the various styles that apply to the text,
         # and so needs to be redone when the style of the text changes.
-        self.paragraphs = self.segment(text.tokens, style, renders, text)
+
+        if splits_from:
+            self.paragraphs = splits_from.paragraphs
+            self.start_segment = splits_from.start_segment
+            self.end_segment = splits_from.end_segment
+            self.has_hyperlinks = splits_from.has_hyperlinks
+            self.hyperlink_targets = splits_from.hyperlink_targets
+            self.has_ruby = splits_from.has_ruby
+        else:
+            self.paragraphs = self.segment(text.tokens, style, renders, text)
 
         first_indent = self.scale_int(style.first_indent)
         rest_indent = self.scale_int(style.rest_indent)
 
+        # True if we've encountered the start and end segments respectively
+        # while assigning times.
+        started = self.start_segment is None
+        ended = False
+
         for p_num, p in enumerate(self.paragraphs):
 
             # RTL - apply RTL to the text of each segment, then
@@ -628,9 +648,9 @@ class Layout(object):
 
             for ts, glyphs in seg_glyphs:
                 # Only assign a time if we're past the start segment.
-                if self.start_segment is not None:
+                if not started:
                     if self.start_segment is ts:
-                        self.start_segment = None
+                        started = True
                     else:
                         continue
 
@@ -681,11 +701,21 @@ class Layout(object):
 
         textsupport.align_and_justify(lines, maxx, style.text_align, style.justify)
 
-        if splits_from and text.style.adjust_spacing:
+        adjust_spacing = text.style.adjust_spacing
+
+        if splits_from and adjust_spacing:
             target_x = self.scale_int(splits_from.size[0] - splits_from.xborder)
             target_y = self.scale_int(splits_from.size[1] - splits_from.yborder)
 
-            textsupport.tweak_glyph_spacing(all_glyphs, lines, target_x - maxx, target_y - y, maxx, y)  # @UndefinedVariable
+            target_x_delta = target_x - maxx
+            target_y_delta = target_y - y
+
+            if adjust_spacing == "horizontal":
+                target_y_delta = 0.0
+            elif adjust_spacing == "vertical":
+                target_x_delta = 0.0
+
+            textsupport.tweak_glyph_spacing(all_glyphs, lines, target_x_delta, target_y_delta, maxx, y)  # @UndefinedVariable
 
             maxx = target_x
             y = target_y
@@ -702,7 +732,7 @@ class Layout(object):
 
         # Place ruby.
         if self.has_ruby:
-            textsupport.place_ruby(all_glyphs, self.scale_int(style.ruby_style.yoffset), sw, sh)
+            textsupport.place_ruby(all_glyphs, self.scale_int(style.ruby_style.yoffset), self.scale_int(style.altruby_style.yoffset), sw, sh)
 
         # Check for glyphs that are being drawn out of bounds, because the font
         # or anti-aliasing or whatever makes them bigger than the bounding box. If
@@ -999,6 +1029,9 @@ class Layout(object):
             elif tag == "color":
                 push().color = renpy.easy.color(value)
 
+            elif tag == "outlinecolor":
+                push().outline_color = renpy.easy.color(value)
+
             elif tag == "alpha":
                 ts = push()
                 if value[0] in "+-":
@@ -1022,6 +1055,15 @@ class Layout(object):
                 ts.ruby_top = True
                 self.has_ruby = True
 
+            elif tag == "art":
+                ts = push()
+                # inherit vertical style
+                vert_style = ts.vertical
+                ts.take_style(style.altruby_style, self)
+                ts.vertical = vert_style
+                ts.ruby_top = "alt"
+                self.has_ruby = True
+
             elif tag == "rb":
                 push().ruby_bottom = True
                 # We only care about ruby if we have a top.
@@ -1177,6 +1219,9 @@ class Layout(object):
 
         for g in l.glyphs:
 
+            if g.time == -1:
+                continue
+
             if g.time > st:
                 continue
 
@@ -1242,7 +1287,11 @@ def layout_cache_clear():
     virtual_layout_cache_new = { }
 
 
-def layout_cache_tick():
+# A list of slow text that's being displayed right now.
+slow_text = [ ]
+
+
+def text_tick():
     """
     Called once per interaction, to merge the old and new layout caches.
     """
@@ -1255,6 +1304,9 @@ def layout_cache_tick():
     virtual_layout_cache_old = layout_cache_new
     virtual_layout_cache_new = { }
 
+    global slow_text
+    slow_text = [ ]
+
 
 VERT_REVERSE = renpy.display.render.Matrix2D(0, -1, 1, 0)
 VERT_FORWARD = renpy.display.render.Matrix2D(0, 1, -1, 0)
@@ -1468,6 +1520,9 @@ class Text(renpy.display.core.Displayable):
         if (self.language != renpy.game.preferences.language) and not self._uses_scope:
             self.set_text(self.text_parameter, substitute=self.substitute, update=True)
 
+        if self.style.slow_abortable:
+            slow_text.append(self)
+
     def set_ctc(self, ctc):
         self.ctc = ctc
         self.dirty = True
@@ -1533,6 +1588,13 @@ class Text(renpy.display.core.Displayable):
         # Text.
         self.tokens, self.displayables = self.get_displayables(tokens)
 
+        for type, text in self.tokens:
+            if type == TAG and text.startswith("a="):
+                self.focusable = True
+                break
+        else:
+            self.focusable = False
+
     def visit(self):
 
         if self.dirty or self.displayables is None:
@@ -1607,6 +1669,12 @@ class Text(renpy.display.core.Displayable):
 
         return rv
 
+    def set_style_prefix(self, prefix, root):
+        if prefix != self.style.prefix:
+            self.kill_layout()
+
+        super(Text, self).set_style_prefix(prefix, root)
+
     def focus(self, default=False):
         """
         Called when a hyperlink gains focus.
@@ -1626,15 +1694,9 @@ class Text(renpy.display.core.Displayable):
         hyperlink_focus = self.style.hyperlink_functions[2]
         target = layout.hyperlink_targets.get(renpy.display.focus.argument, None)
 
-        if hyperlink_focus:
+        if hyperlink_focus and (not default) and (target is not None):
             return hyperlink_focus(target)
 
-    def set_style_prefix(self, prefix, root):
-        if prefix != self.style.prefix:
-            self.kill_layout()
-
-        super(Text, self).set_style_prefix(prefix, root)
-
     def unfocus(self, default=False):
         """
         Called when a hyperlink loses focus, or isn't focused to begin with.
@@ -1645,7 +1707,7 @@ class Text(renpy.display.core.Displayable):
 
         hyperlink_focus = self.style.hyperlink_functions[2]
 
-        if hyperlink_focus:
+        if hyperlink_focus and not default:
             return hyperlink_focus(None)
 
     def call_slow_done(self, st):
@@ -1677,14 +1739,24 @@ class Text(renpy.display.core.Displayable):
         """
 
         if self.slow and renpy.display.behavior.map_event(ev, "dismiss") and self.style.slow_abortable:
-            self.call_slow_done(st)
-            self.slow = False
+
+            for i in slow_text:
+                if i.slow:
+                    i.call_slow_done(st)
+                    i.slow = False
+
             raise renpy.display.core.IgnoreEvent()
 
         layout = self.get_layout()
+
         if layout is None:
             return
 
+        if layout.redraw_typewriter(st) is None:
+            if self.slow:
+                self.call_slow_done(st)
+                self.slow = False
+
         for d, xo, yo, _ in layout.displayable_blits:
             rv = d.event(ev, x - xo - layout.xoffset, y - yo - layout.yoffset, st)
             if rv is not None:
@@ -1902,7 +1974,7 @@ class Text(renpy.display.core.Displayable):
             if redraw is not None:
                 renpy.display.render.redraw(self, redraw)
             else:
-                self.call_slow_done(st)
+                renpy.display.interface.timeout(0)
 
         rv.forward = layout.forward
         rv.reverse = layout.reverse
